using System;
using static System.Console;
using static System.Math;

class Board {
  public int[] square;   // 0 = empty, 1 = X, 2 = O
  int moves = 0;  // how many moves so far
  
  public Board(int n) { square = new int[n]; }
  
  public int size => square.Length;
  
  // Return 1 if X wins, -1 if O wins, 0 if a draw.
  public int? result() {
    if (moves < size) return null;
    
    int streak = 0;
    int longest_x = 0, longest_o = 0;
    
    for (int i = 0 ; i < size ; ++i) {
      if (i > 0 && square[i] == square[i - 1])
        streak += 1;
      else
        streak = 1;
        
      if (square[i] == 1)
        longest_x = Max(longest_x, streak);
      else longest_o = Max(longest_o, streak);
    }
    return Sign(longest_x - longest_o);
  }

  // Make this player move at square i
  public bool move(int player, int i) {
    if (square[i] == 0) {
      square[i] = player;
      ++moves;
      return true;
    } else return false;  // invalid move
  }
  
  // Undo a move at square i
  public void unmove(int i) {
    square[i] = 0;
    --moves;
  }
}

class Minimax {
  // Return the best possible score for the current player.
  public static int minimax(Board b, int player) {  // player = 1 or 2
    if (b.result() is int r)
      return r;  // game is done
    
    int m = player == 1 ? int.MinValue : int.MaxValue;
    for (int i = 0 ; i < b.size ; ++i)  // for every possible move
      if (b.move(player, i)) {  // move is valid
        int n = minimax(b, 3 - player  /* other player */);
        b.unmove(i);
        
        if (player == 1) {  // maximizing
          if (n > m) {
            m = n;
          }
        } else { // player 2
          if (n < m) {
            m = n;
          }
        }
      }  // end if
    return m;
  }
}

class Game {
  static void Main() {
    for (int size = 1 ; size <= 12 ; ++size) {
      DateTime start = DateTime.Now;
      int result = Minimax.minimax(new Board(size), 1);
      int ms = (int) (DateTime.Now - start).TotalMilliseconds;
      WriteLine($"{size} -> {result} ({ms} ms)");
    }
  }
}
